import { basename, dirname } from './path'
let fs: any = {}
let xlsx: any = {}

if (window.nodeRequire) {
  const require1 = window.nodeRequire
  fs = require1('fs')
  xlsx = require1('xlsx')
}

const varNameRow = 4
const typeNameRow = 3
const dataBeginRow = 5
const csNameRow = 2

declare global {
  interface Window {
    uploadFile: (path: string, showPath: string)=>void
    nodeRequire: (path: string) => any
  }
}

export class JCExcelParse {
  private filePath: string
  private jsonFile = ''
  private jsonPath: string
  private gameId: string
  private worksheet!: any

  constructor(gameId: string, filePath: string) {
    this.filePath = filePath
    this.gameId = gameId
    this.jsonPath = this.updateJsonFilePath()
    this.readExcel()
  }

  readExcel() {
    const arraybuffer = fs.readFileSync(this.filePath)
    const data = new Uint8Array(arraybuffer)
    const arr = []
    for (let i = 0; i !== data.length; ++i) arr[i] = String.fromCharCode(data[i])
    const bstr = arr.join('')
    /* Call XLSX */
    const workbook = xlsx.read(bstr, { type: 'binary' })

    const firstSheetName = workbook.SheetNames[0]
    this.worksheet = workbook.Sheets[firstSheetName]
    console.log(this.worksheet)
  }

  private updateJsonFilePath() {
    const baseName = basename(this.filePath)
    const jsonFile = JCExcelParse.parseFilename(baseName)
    this.jsonFile = jsonFile
    const cfgPath = `${dirname(this.filePath)}/cfgs`
    if (!fs.existsSync(cfgPath)) {
      fs.mkdirSync(cfgPath)
    }
    return `${dirname(this.filePath)}/cfgs/${jsonFile}`
  }

  toCfgJson() {
    const data = this.parseOneSheet(this.worksheet)
    this.writeJson(this.jsonPath, data)
  }

  public uploadJson() {
    const showPath = `${this.gameId}/cfgs/${this.jsonFile}`
    window.uploadFile(this.jsonPath, showPath)
  }

  public static parseFilename(baseName: string) {
    return baseName.replace(/\d+?_/, '') + '_tbl.json'
  }

  private parseDataRange(refStr: string) {
    const strs = refStr.split(':')
    const bX = this.posToNum(strs[0].replace(/\d/g, ''))
    const bY = +strs[0].replace(/[A-Z]/g, '')
    const eX = this.posToNum(strs[1].replace(/\d/g, ''))
    const eY = +strs[1].replace(/[A-Z]/g, '')
    return { bX, bY, eX, eY }
  }

  private numToPos(n: number) {
    const ordA = 'A'.charCodeAt(0)
    const ordZ = 'Z'.charCodeAt(0)
    const len = ordZ - ordA + 1
    let s = ''
    while (n >= 0) {
      s = String.fromCharCode(n % len + ordA) + s
      n = Math.floor(n / len) - 1
    }
    return s
  }

  private posToNum(ord: string) {
    const ordA = 'A'.charCodeAt(0)
    let result = 0
    for (let i = 0; i < ord.length; i++) {
      const v = ord.charCodeAt(ord.length - i - 1) - ordA + 1
      result += (v * Math.pow(26, i))
    }
    return (result | 0) - 1
  }

  private getTypeKey(col: string) {
    return `${col}${typeNameRow}`
  }

  private getNameKey(col: string) {
    return `${col}${varNameRow}`
  }

  private getCSKey(col: string) {
    return `${col}${csNameRow}`
  }

  private parseKvVal(val: string) {
    const arr = val.split('|')
    const result: any = {}
    if (!val) {
      return result
    }
    for (const subval of arr) {
      const subArr = subval.split(',')
      if (subArr.length < 2) {
        continue
      }
      if (isNaN(Number(subArr[1]))) {
        result[subArr[0]] = subArr[1]
      } else {
        result[subArr[0]] = +subArr[1]
      }
    }
    return result
  }

  private parseObjListVal(val: string) {
    const result: any[] = []
    if (!val) {
      return result
    }
    for (const subval of val.split('|')) {
      const olst = subval.split(',')
      const obj: any = {}
      for (const ostr of olst) {
        if (!ostr) {
          continue
        }
        const subArr = ostr.split(':')
        if (subArr.length < 2) {
          continue
        }
        if (isNaN(Number(subArr[1]))) {
          obj[subArr[0]] = subArr[1]
        } else {
          obj[subArr[0]] = +subArr[1]
        }
      }
      result.push(obj)
    }
    return result
  }

  private parseOneSheet(sheet: any) {
    const { bX, bY, eX, eY } = this.parseDataRange(sheet['!ref']!)
    const results = []
    for (let i = dataBeginRow; i < eY; i++) {
      const data: any = {}
      for (let j = bX; j < eX; j++) {
        const key = this.numToPos(j)
        const cs = (sheet[this.getCSKey(key)])?.v || ''
        if (!cs || !(cs === 'c' || cs === 'a')) {
          continue
        }
        const fieldName = (sheet[this.getNameKey(key)])?.v || ''
        if (!fieldName) {
          continue
        }
        const type = (sheet[this.getTypeKey(key)])?.v || 'string'
        const fieldVal = (sheet[`${key}${i}`])?.v
        switch (type) {
          case 'number':
            data[fieldName] = +fieldVal || 0
            break
          case 'bool':
            data[fieldName] = !!fieldVal
            break
          case 'float':
            data[fieldName] = fieldVal || '0.0'
            break
          case 'string_list':
            data[fieldName] = fieldVal ? (fieldVal as string).split('|') : []
            break
          case 'float_list':
          case 'number_list':
            if (fieldVal) {
              data[fieldName] = (fieldVal as string).split('|').map(o => +o)
            }
            break
          case 'kv_list':
            data[fieldName] = this.parseKvVal(fieldVal)
            break
          case 'obj_list':
            data[fieldName] = this.parseObjListVal(fieldVal)
            break
          default:
            data[fieldName] = fieldVal !== undefined ? fieldVal + '' : ''
            break
        }
      }
      results.push(data)
    }
    console.dir(results)
    return results
  }

  private writeJson(fileName: string, data: any) {
    fs.writeFileSync(fileName, JSON.stringify(data))
  }
}
