const EXTNAME_RE = /(\.[^\.\/\?\\]*)(\?.*)?$/
const DIRNAME_RE = /((.*)(\/|\\|\\\\))?(.*?\..*$)?/
const NORMALIZE_RE = /[^\.\/]+\/\.\.\//

/**
     * !#en Join strings to be a path.
     * !#zh 拼接字符串为 Path
     * @method join
     * @example {@link cocos2d/core/utils/CCPath/join.js}
     * @returns {String}
     */
export const join = function(...args: any[]) {
  const l = args.length
  let result = ''
  for (let i = 0; i < l; i++) {
    result = (result + (result === '' ? '' : '/') + args[i]).replace(/(\/|\\\\)$/, '')
  }
  return result
}

/**
     * !#en Get the ext name of a path including '.', like '.png'.
     * !#zh 返回 Path 的扩展名，包括 '.'，例如 '.png'。
     * @method extname
     * @example {@link cocos2d/core/utils/CCPath/extname.js}
     * @param {String} pathStr
     * @returns {*}
     */
export const extname = function(pathStr: string) {
  const temp = EXTNAME_RE.exec(pathStr)
  return temp ? temp[1] : ''
}

/**
     * !#en Get the main name of a file name
     * !#zh 获取文件名的主名称
     * @method mainFileName
     * @param {String} fileName
     * @returns {String}
     * @deprecated
     */
export const mainFileName = function(fileName: string) {
  if (fileName) {
    const idx = fileName.lastIndexOf('.')
    if (idx !== -1) { return fileName.substring(0, idx) }
  }
  return fileName
}

/**
* !#en Get the file name of a file path.
* !#zh 获取文件路径的文件名。
* @method basename
* @example {@link cocos2d/core/utils/CCPath/basename.js}
* @param {String} pathStr
* @param {String} [extname]
* @returns {*}
*/
export const basename = function(pathStr: string, extName?: string): string {
  const index = pathStr.indexOf('?')
  if (index > 0) pathStr = pathStr.substring(0, index)
  const reg = /(\/|\\)([^\/\\]+)$/g
  const result = reg.exec(pathStr.replace(/(\/|\\)$/, ''))
  if (!result) return ''
  const baseName = result[2]
  extName = extName || extname(baseName)
  if (pathStr.substring(pathStr.length - extName.length).toLowerCase() === extName.toLowerCase()) {
    return baseName.substring(0, baseName.length - extName.length)
  }
  return baseName
}

/**
* !#en Get dirname of a file path.
* !#zh 获取文件路径的目录名。
* @method dirname
* @example {@link cocos2d/core/utils/CCPath/dirname.js}
* @param {String} pathStr
* @returns {*}
*/
export const dirname = function(pathStr: string) {
  const temp = DIRNAME_RE.exec(pathStr)
  return temp ? temp[2] : ''
}

/**
* !#en Change extname of a file path.
* !#zh 更改文件路径的扩展名。
* @method changeExtname
* @example {@link cocos2d/core/utils/CCPath/changeExtname.js}
* @param {String} pathStr
* @param {String} [extname]
* @returns {String}
*/
export const changeExtname = function(pathStr: string, extname: string) {
  extname = extname || ''
  let index = pathStr.indexOf('?')
  let tempStr = ''
  if (index > 0) {
    tempStr = pathStr.substring(index)
    pathStr = pathStr.substring(0, index)
  }
  index = pathStr.lastIndexOf('.')
  if (index < 0) return pathStr + extname + tempStr
  return pathStr.substring(0, index) + extname + tempStr
}
/**
* !#en Change file name of a file path.
* !#zh 更改文件路径的文件名。
* @example {@link cocos2d/core/utils/CCPath/changeBasename.js}
* @param {String} pathStr
* @param {String} basename
* @param {Boolean} [isSameExt]
* @returns {String}
*/
export const changeBasename = function(pathStr: string, basename: string, isSameExt: boolean) {
  if (basename.indexOf('.') === 0) return changeExtname(pathStr, basename)
  let index = pathStr.indexOf('?')
  let tempStr = ''
  const ext = isSameExt ? extname(pathStr) : ''
  if (index > 0) {
    tempStr = pathStr.substring(index)
    pathStr = pathStr.substring(0, index)
  }
  index = pathStr.lastIndexOf('/')
  index = index <= 0 ? 0 : index + 1
  return pathStr.substring(0, index) + basename + ext + tempStr
}
// todo make public after verification
const _normalize = function(url: string) {
  let oldUrl = url = String(url)

  // removing all ../
  do {
    oldUrl = url
    url = url.replace(NORMALIZE_RE, '')
  } while (oldUrl.length !== url.length)
  return url
}

// @param {string} path
export const stripSep = function(path: string) {
  return path.replace(/[\/\\]$/, '')
}
